import random


class RandomHelper():
    def __init__(self, config, storageAdmin):
        self.config = config
        self.storageAdmin = storageAdmin
        self.excludeList = config.randomizer.exclude
        random.seed()

    def get_counters(self):
        return self.storageAdmin.get_counters()

    def get_random_number(self):
        """
        Returns a random number to pick from
        among the available indexed posts.

        It excludes the post indexes that are in
        the exclude list in the configuration file.
        """
        start, last, previous = self.get_counters()
        ret = random.randint(start, last)
        while ret in self.excludeList:
            ret = random.randint(start, last)
        return ret

    def should_i_run(self):
        ont = self.config.randomizer.ontimes
        allt = self.config.randomizer.outoftimes
        # assumes ont is always less than or equal to allt
        assert(ont <= allt)
        return ont >= random.randint(1, allt)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for `infobot` package.
Module tested:  storage
"""

import pytest

from infobot.config import Admin as ConfigAdmin
from infobot.storage.file import FileAdmin as FileAdmin
from infobot.brains import Brains


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string


def test_storage():
    config = ConfigAdmin("./config.yaml")
    fa = FileAdmin(config, config.storageadmindetails)
    assert(fa._directory == Brains.expand_home(
        "~/Nextcloud/Documents/rusty_robot/data/"))
    assert(fa._counterfile == Brains.expand_home(
           "~/Nextcloud/Documents/rusty_robot/data/counter.yaml"))




class Admin():
    def __init__(self, config):
        "abstract"
        self.config = config
        pass

    def status(self):
        raise NotImplementedError()

    def store_all(self, fromdir, topic):
        raise NotImplementedError()

    def store(self, data, toIndex):
        raise NotImplementedError()

    def read_from(self, index):
        raise NotImplementedError()

    def get_counters(self):
        raise NotImplementedError()

    def increment_last(self):
        raise NotImplementedError()

    def format_index(self, topicname, num):
        raise NotImplementedError()

    def get_header(self, socialnetwork, topic, num):
        raise NotImplementedError()

    def get_footer(self, socialnetwork, topic, num):
        raise NotImplementedError()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for `infobot` package.
Module tested:  config
"""

import pytest

from infobot.config import Admin as ConfigAdmin

# from click.testing import CliRunner
# from infobot import cli


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string


def test_config():
    config = ConfigAdmin("./config.yaml")
    assert(config.topic.name == "rust")
    assert(config.dev.storageclass == "FileAdmin")
    assert(config.randomizer.ontimes == 24)
    assert(config.randomizer.outoftimes == 24)
    # assert(config.topic.opt == "")  # optional field



import os
from mastodon import Mastodon

import infobot.konstants as K
from infobot.brains import Brains
from infobot.social.template import SocialPlugin


class MastodonPluginConf():
    def __init__(self, filedata):
        "Handles the configuation data for Mastodon plugin"
        self.socialAppName = filedata[K.socialAppKey]
        self.clientAppName = filedata[K.clientAppNameKey]
        self.apiURL = filedata[K.apiURLKey]
        self.clientSecretFilename = filedata[K.clientSecretKey]
        self.userSecretFilename = filedata[K.userSecretKey]


class MastodonPlugin(SocialPlugin):
    def __init__(self, config, socialplugindetails, storageadmin):
        """
        This is a reference class for a social plugin
        It does not post to any social network
        It simply outputs to the stdout as if it is posting
        """
        details = MastodonPluginConf(socialplugindetails)
        super().__init__(config, details.socialAppName)
        self._details = details
        self._clientappname = self._details.clientAppName
        self._apiURL = self._details.apiURL
        self._clientSecretFilename = Brains.expand_home(
            self._details.clientSecretFilename)
        self._userSecretFilename = Brains.expand_home(
            self._details.userSecretFilename)
        self.storageAdmin = storageadmin

    def status(self):
        print("Mastodon social plugin is active with appname {}".format(
            self._clientappname
        ))
        print("  This client is registered: {}".format(self.registered()))
        print("  This user is logged in: {}".format(self.logged_in()))

    def register(self):
        # Register app - only once!
        Mastodon.create_app(
            self._clientappname,
            api_base_url=self._apiURL,
            to_file=self._clientSecretFilename
        )
        # this also logs in making the app ready for posting
        if self.login():
            if self.logged_in():
                print("One time registration succesfully completed")
                return True
        else:
            return False

    def login(self):
        if not self.registered():
            print(
                ("{} requires a one time registration, "
                 "please run with '-r' flag first").format(
                    self.socialName))
            return False
        if self.logged_in():
            return True
        # login and save login key
        mastodon = Mastodon(
            client_id=self._clientSecretFilename,
            api_base_url=self._apiURL
        )
        uid = str(input("Enter mastodon userid email (e.g abc@xyz.com): "))
        pwd = str(input("Enter password for user {}: ".format(uid)))
        mastodon.log_in(
            uid,
            pwd,
            to_file=self._userSecretFilename
        )
        return True

    def logout(self):
        pass

    def registered(self):
        # ensure client secret file is in place
        # and it is not 0 length
        if os.path.exists(self._clientSecretFilename):
            if os.path.isfile(self._clientSecretFilename):
                if os.path.getsize(self._clientSecretFilename) > 0:
                    return True
        return False

    def logged_in(self):
        # ensure user secret file is in place
        # and it is not 0 length
        if os.path.exists(self._userSecretFilename):
            if os.path.isfile(self._userSecretFilename):
                if os.path.getsize(self._userSecretFilename) > 0:
                    return True
        return False

    def post(self, topic, num, data):
        if not self.logged_in():
            self.login()
            if not self.logged_in():
                assert(False)
        hdr = self.storageAdmin.get_header(self.socialName, topic, num)
        ftr = self.storageAdmin.get_footer(self.socialName, topic, num)
        postdata = "{}\n{}\n{}".format(hdr, data, ftr)
        # Create actual API instance
        mastodon = Mastodon(
            access_token=self._userSecretFilename,
            api_base_url=self._apiURL
        )
        mastodon.toot(postdata)
        return True

    def list_followers(self):
        return["a", "b", "c"]

    def follow(self, other):
        print("following @"+other)



import argparse
import importlib
import os

import infobot.konstants as K
from infobot.config import Admin as ConfigAdmin
from infobot.index import RandomHelper


class Brains():
    """
    Brains does the main coordination for infobot.
    It works with social plugin and the storage admin
    to make the posts or add more data for future posts
    from a user supplied directory.
    """

    def __init__(self):
        """
        create a coordinator for infobot
        and start him on its tasks based on
        user supplied command line arguments
        """
        args = Brains.process_args()
        configFileName = Brains._get_config_file_name(args)
        self.config = ConfigAdmin(configFileName)
        StorageAdmin = self.resolve_storage_admin()
        self.storageAdmin = StorageAdmin(
            self.config, self.config.storageadmindetails)
        self.randomHelper = RandomHelper(self.config, self.storageAdmin)
        SocialPlugin = self.resolve_social_plugin()
        self.socialPlugin = SocialPlugin(
            self.config, self.config.socialplugindetails, self.storageAdmin)
        self.wakeup(args)

    def wakeup(self, args):
        """
        There are two modes of operations
        Main mode is to wake up and make a post to a social network
        The other one is when future posts are added to a storage destination
        """
        if args.addfrompath is not None:
            self.add_future_posts(args.addfrompath)
        elif args.liststatus:
            self.display_status()
        elif args.registerapp:
            self.register_app()
        else:
            if self.randomHelper.should_i_run():
                self.post_to_social()

    def display_status(self):
        self.config.status()
        self.storageAdmin.status()
        self.socialPlugin.status()

    def post_to_social(self):
        """
        This is the method that organizes the social network interaction
        """
        num = self.randomHelper.get_random_number()
        index = self.storageAdmin.format_index(
            self.config.topic.name,
            num)
        filedata = self.storageAdmin.read_from(index)
        self.socialPlugin.login()
        self.socialPlugin.post(self.config.topic.name,
                               num, filedata)
        self.socialPlugin.logout()

    def add_future_posts(self, frompath):
        self.storageAdmin.store_all(frompath,
                                    self.config.topic.name)

    def register_app(self):
        self.socialPlugin.register()

    def resolve_storage_admin(self):
        storageModuleObj = Brains._module(K.packageName, K.storageModule,
                                          self.config.dev.storagemodule)
        storageClassObj = getattr(
            storageModuleObj, self.config.dev.storageclass)
        return storageClassObj

    def resolve_social_plugin(self):
        socialModuleObj = Brains._module(K.packageName, K.socialModule,
                                         self.config.dev.socialmodule)
        socialPluginClassObj = getattr(
            socialModuleObj, self.config.dev.socialclass)
        return socialPluginClassObj

    @staticmethod
    def process_args():
        """
        handles command line options and flags
        """
        example_usage_text = '''Example:

        ./bot.py  -c ./config.yaml

        '''
        parser = argparse.ArgumentParser(
            description="Wakes up infobot to post",
            epilog=example_usage_text,
            formatter_class=argparse.RawDescriptionHelpFormatter)
        parser.add_argument("-c", "--confpath",
                            help="path for yaml configuration file",
                            type=str, default="./config.yaml")
        parser.add_argument("-a", "--addfrompath",
                            help="move entries from a directory" +
                            " to bot storage",
                            type=str, required=False)
        parser.add_argument("-r", "--registerapp",
                            help="one time app registration is required" +
                            " for some social plugins",
                            action="store_true",
                            default=False, required=False)
        parser.add_argument("-l", "--liststatus",
                            help="list current settings and status " +
                            "for some social plugins",
                            action="store_true",
                            default=False, required=False)
        args = parser.parse_args()
        return args

    @staticmethod
    def _get_config_file_name(args):
        return Brains.expand_home(args.confpath)

    @staticmethod
    def _module(packagename, dirname, modulename):
        return importlib.import_module(
            packagename + "." +
            dirname + modulename, package=packagename)

    @staticmethod
    def expand_home(origPath):
        if "~" in origPath:
            origPath = origPath.replace("~",
                                        str(os.path.expanduser('~')))
        return origPath



import os
import yaml

import infobot.konstants as K
from infobot.storage.template import Admin
from infobot.config import Admin as ConfigAdm
from infobot.brains import Brains


class FileAdminConf():
    def __init__(self, fileadmindetails):
        "Configurations Object for File based storage admin"
        self.directory = Brains.expand_home(
            fileadmindetails[K.dataDirectoryKey])
        self.counterfile = Brains.expand_home(
            fileadmindetails[K.counterKey])
        self.indexFileFormat = fileadmindetails[K.indexFileFormatKey]


class FileAdmin(Admin):
    def __init__(self, config, fileadmindetails):
        """
        This is a file system based storage admin.
        It stores each information to be posted as a separate file
        It also keeps track of the number of items to be posted
        """
        super().__init__(config)
        self._details = FileAdminConf(fileadmindetails)
        self._directory = self._details.directory
        self._counterfile = self._details.counterfile
        self._indexFileFormat = self._details.indexFileFormat

    def status(self):
        print("File Based Storage Admin is active")
        print("  Reading posts from: {}".format(
            self._directory
        ))
        print("  Counter file at: {}".format(
            self._counterfile
        ))
        start, last, previous = self.get_counters()
        print("  There are {} posts, last post index is: {}".format(
            str(last - start + 1),
            str(last)
        ))

    def format_index(self, topic, num):
        """
        returns formatted  filename
        """
        return self._indexFileFormat.format(topic, str(num))

    def read_from(self, index):
        """
        Returns the postable data at the index location
        """
        fullpath = os.path.join(self._directory, index)
        with open(fullpath) as postfile:
            postdata = postfile.read()
        return postdata

    def get_counters(self):
        counterData = ConfigAdm.read_yaml(self._counterfile)
        return (int(counterData[K.startKey]),
                int(counterData[K.lastKey]),
                int(counterData[K.previousKey]))

    def increment_last(self):
        with open(self._counterfile, "r") as f:
            data = yaml.safe_load(f)
            cl = int(data[K.lastKey])
            data[K.lastKey] = cl + 1
        with open(self._counterfile, 'w') as f:
            yaml.safe_dump(data, f, default_flow_style=False)

    def store_all(self, fromdir, topic):
        print(("Will move files with name containing {}\n"
               "from: {}\n"
               "into: {}")
              .format(self.config.topic.name,
                      fromdir,
                      self._directory))
        fromdir = Brains.expand_home(fromdir)
        for f in os.listdir(fromdir):
            sourceFile = os.path.join(fromdir, f)
            print(sourceFile)
            if os.path.isdir(f) or (topic not in str(f)):
                continue
            with open(sourceFile) as fd:
                data = fd.read()
                self.store(topic, data)
                os.remove(sourceFile)

    def store(self, topic, data):
        _, last, _ = self.get_counters()
        num = last+1
        filename = self.format_index(topic, num)
        fullpath = os.path.join(self._directory, filename)
        with open(fullpath, "w") as f:
            f.write(data)
            self.increment_last()

    def get_header(self, socialNetwork, topic, num):
        header = FileAdmin.templates[socialNetwork][K.headerKey]
        return header.format(topic, num)

    def get_footer(self, socialNetwork, topic, num):
        footer = FileAdmin.templates[socialNetwork][K.footerKey]
        return footer.format(topic, num)

    templates = {
        K.fakeKey:
        {
            K.headerKey: """
            #{}-{}""",

            K.footerKey: """
------
Please reply to report error"""

        },
        K.mastodonKey:
        {
            K.headerKey:  "#{}-{}\n",

            K.footerKey: ""
        }

    }



import yaml


class Topic():
    def __init__(self, filedata):
        """
        Simple configuration file helper: Topic details
        """
        self.name = filedata["name"]


class Randomizer():
    def __init__(self, filedata):
        """
        Simple configuration file helper: Random posting behavior control
        """
        self.ontimes = int(filedata["ontimes"])
        self.outoftimes = int(filedata["outoftimes"])
        self.start = int(filedata["start"])
        excludeList = filedata["exclude"].split(",")
        self.exclude = [int(i) for i in excludeList]


class Dev():
    def __init__(self, filedata):
        """
        Only for developers
        Use this configuration helper to extend infobot
        """
        self.storagemodule = filedata["storagemodule"]
        self.storageclass = filedata["storageclass"]
        self.socialmodule = filedata["socialmodule"]
        self.socialclass = filedata["socialclass"]
        # sample optional argument
        # self.opt = filedata.get("opt", "")


class Admin():
    def __init__(self, filepath):
        """
        This is the main configuration admin class.
        It parses the  configuration from a yaml file.
        The file name is given as a commandline argument to the prog.
        """
        filedata = Admin.read_yaml(filepath)
        self.configfilepath = filepath
        self.topic = Topic(filedata["topic"])
        self.dev = Dev(filedata["dev"])
        self.randomizer = Randomizer(filedata["randomizer"])
        self.storageadmindetails = filedata[self.dev.storageclass
                                            + "Details"]
        self.socialplugindetails = filedata[self.dev.socialclass
                                            + "Details"]

    def status(self):
        print("Infobot instance for topic: {}".format(self.topic.name))
        print("  Configuration file: {}".format(self.configfilepath))
        print("  Posts {} out of {} times".format(
            self.randomizer.ontimes,
            self.randomizer.outoftimes
        ))
        print("  Starting from: {} and excluding: {}".format(
            self.randomizer.start,
            self.randomizer.exclude
        ))

    @staticmethod
    def read_yaml(yamlFilePath):
        with open(yamlFilePath) as yf:
            data = yaml.safe_load(yf.read())
        return data



class SocialPlugin():
    def __init__(self, config, name):
        "abstract"
        self.config = config
        self.socialName = name

    def status(self):
        raise NotImplementedError()

    def login(self):
        raise NotImplementedError()

    def logout(self):
        raise NotImplementedError()

    def post(self, topic, num, data):
        raise NotImplementedError()

    def list_followers(self):
        raise NotImplementedError()

    def follow(self, other):
        raise NotImplementedError()



import infobot.konstants as K
from infobot.social.template import SocialPlugin


class FakeSocialPluginConf():
    def __init__(self, filedata):
        "Handles the configuation data for social plugin"
        self.userid = filedata[K.useridKey]
        self.password = filedata[K.passwdKey]
        self.socialAppName = filedata[K.socialAppKey]


class FakeSocialPlugin(SocialPlugin):
    def __init__(self, config, socialplugindetails, storageadmin):
        """
        This is a reference class for a social plugin
        It does not post to any social network
        It simply outputs to the stdout as if it is posting
        """
        details = FakeSocialPluginConf(socialplugindetails)
        super().__init__(config, details.socialAppName)
        self._details = details
        self._userid = self._details.userid
        self._password = self._details.password
        self.storageAdmin = storageadmin

    def status(self):
        print("Fake social Plugin is active")
        print("  This only prints to stdout, nothing else.")

    def register(self):
        print("No need to register")
        return True

    def login(self):
        print("Logging into fake with {}/{}".
              format(self._userid, self._password))
        return True

    def logout(self):
        print("Logging out from fake.")
        return True

    def post(self, topic, num, data):
        hdr = self.storageAdmin.get_header(self.socialName, topic, num)
        ftr = self.storageAdmin.get_footer(self.socialName, topic, num)
        postdata = "{}\n{}\n{}".format(hdr, data, ftr)
        print(postdata)
        return postdata

    def list_followers(self):
        return["a", "b", "c"]

    def follow(self, other):
        print("following @"+other)



# k_configfile = "configFileName"

# dev keys
packageName = "infobot"
socialModule = "social."
storageModule = "storage."

# storage keys
dataDirectoryKey = "directory"
counterKey = "counterfile"
indexFileFormatKey = "indexformat"

# index related keys
startKey = "start"
lastKey = "last"
previousKey = "previous"

# post format keys
footerKey = "footer"
headerKey = "header"
fakeKey = "fake"
mastodonKey = "mastodon"

# social Keys
useridKey = "userid"
passwdKey = "password"

socialAppKey = "socialapp"
apiURLKey = "apiurl"
clientAppNameKey = "clientappname"
clientSecretKey = "clientsecretfilename"
userSecretKey = "usersecretfilename"
